package com.zanxiang.game.gs.data.serve.service.impl;


import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import com.mybatisflex.core.row.Row;
import com.zanxiang.game.gs.data.serve.pojo.dto.RolePayDTO;
import com.zanxiang.game.gs.data.serve.pojo.vo.PageVO;
import com.zanxiang.game.gs.data.serve.pojo.vo.RolePayVO;
import com.zanxiang.game.gs.data.serve.service.ISysUserService;
import com.zanxiang.module.util.DateUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;
import com.zanxiang.game.gs.data.serve.service.IAdsRoleAmountRateParentService;
import com.zanxiang.game.gs.data.serve.pojo.entity.game.parent.AdsRoleAmountRateParent;
import com.zanxiang.game.gs.data.serve.dao.mapper.game.parent.AdsRoleAmountRateParentMapper;
import com.mybatisflex.spring.service.impl.ServiceImpl;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.zanxiang.game.gs.data.serve.pojo.entity.game.parent.table.AdsRoleAmountDataParentTableDef.ADS_ROLE_AMOUNT_DATA_PARENT;
import static com.zanxiang.game.gs.data.serve.pojo.entity.game.parent.table.AdsRoleAmountRateParentTableDef.ADS_ROLE_AMOUNT_RATE_PARENT;

/**
 * 游戏GS管理-GS角色付费留存 服务层实现。
 *
 * @author shaosong
 * @since 1.0
 */
@Service
public class AdsRoleAmountRateParentServiceImpl extends ServiceImpl<AdsRoleAmountRateParentMapper, AdsRoleAmountRateParent> implements IAdsRoleAmountRateParentService {
    @Resource
    private ISysUserService sysUserService;

    @Override
    public PageVO<RolePayVO, Row> rolePayOfPage(RolePayDTO dto) {
        QueryWrapper rolePayWrapper = getRolePayWrapper(dto);
        Page<RolePayVO> paginate = getMapper().paginateAs(dto.getPageNum(), dto.getPageSize(), rolePayWrapper, RolePayVO.class);
        if (0 == paginate.getTotalRow()) {
            return PageVO.empty(dto);
        }
        List<RolePayVO> records = paginate.getRecords();
        Set<Long> gsIdList = records.stream().map(RolePayVO::getGsId).collect(Collectors.toSet());
        //GS名称查询
        Map<Long, String> sysUserMap = CollectionUtils.isEmpty(gsIdList) ? null : sysUserService.getSysUserMap(gsIdList);
        records.forEach(item -> {
            item.setGsName(sysUserMap != null ? sysUserMap.get(item.getGsId()) : null);
            item.setDtDiff(DateUtil.intervalOfDays(item.getDt(), LocalDate.now()));
        });
        return new PageVO<>(paginate);
    }

    private QueryWrapper getRolePayWrapper(RolePayDTO dto) {
        return QueryWrapper.create()
                .select(ADS_ROLE_AMOUNT_RATE_PARENT.ALL_COLUMNS)
                .ge(AdsRoleAmountRateParent::getDt, dto.getDtBegin(), null != dto.getDtBegin())
                .le(AdsRoleAmountRateParent::getDt, dto.getDtEnd(), null != dto.getDtEnd())
                .in(AdsRoleAmountRateParent::getGsId, dto.getGsIdList(), CollectionUtils.isNotEmpty(dto.getGsIdList()))
                .orderBy(dto.getSortFiled(), dto.getSortAsc());
    }
}